
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.002004
eval "use Test::Spelling 0.12; use Pod::Wordlist; 1" or die $@;

set_spell_cmd( 'aspell list' );
add_stopwords( <DATA> );
all_pod_files_spelling_ok( 'bin', 'lib' );
__DATA__
LICENCE
STDERR
STDOUT
ipadd
params
subcommand
subcommands
CTID
applyconfig
avnumproc
bootorder
chkpnt
cpulimit
cpumask
cpus
cpuunits
ctid
dcachesize
devnodes
dgramrcvbuf
diskinodes
diskspace
hostname
ioprio
ipdel
ips
iptables
kmemsize
lockedpages
nameserver
noatime
numfile
numflock
numiptent
numothersock
numproc
numpty
numsiginfo
numtcpsock
onboot
oomguarpages
ostemplate
othersockbuf
physpages
privvmpages
quotainit
quotaoff
quotaon
quotatime
quotaugidlimit
runscript
searchdomain
setmode
shmpages
swappages
tcprcvbuf
tcpsndbuf
umount
userpasswd
vmguarpages
Alan
Young
lib
OpenVZ
Vzlist
Vzctl
